@@ -1,20 +1,23 @@
-try:
-    # Attempt to use regular keras first, as we figure it's installed for a reason
-    import keras
-except ImportError:
-    from tensorflow import keras
+import logging
 
+try:
+    import keras as keras_keras
+except ImportError:
+    keras_keras = None
+try:
+    from tensorflow import keras as tf_keras
+except ImportError:
+    tf_keras = None
 
 from dlhub_sdk.models.servables.python import BasePythonServableModel
+from dlhub_sdk.models.servables import ArgumentTypeMetadata
 from dlhub_sdk.utils.types import compose_argument_block
 
-
-keras_version = keras.__version__
-_keras_version_tuple = tuple(int(i) for i in keras_version.rstrip("-tf").split("."))
+logger = logging.getLogger(__name__)
 _summary_limit = 10000
 
 
-def _detect_backend(output):
+def _detect_backend(keras, output):
     """Add the backend
 
     Args:
@@ -35,7 +38,7 @@ class KerasModel(BasePythonServableModel):
 
     @classmethod
     def create_model(cls, model_path, output_names=None, arch_path=None,
-                     custom_objects=None):
+                     custom_objects=None, force_tf_keras: bool = False) -> 'KerasModel':
         """Initialize a Keras model.
 
         Args:
@@ -48,8 +51,26 @@ class KerasModel(BasePythonServableModel):
                 `Keras Documentation
                 <https://www.tensorflow.org/api_docs/python/tf/keras/models/load_model>`_
                 for more details.
+            force_tf_keras (bool): Force the use of TF.Keras even if keras is installed
        """
-        output = super(KerasModel, cls).create_model('predict')
+        output: KerasModel = super().create_model('predict')
+        if force_tf_keras:
+            if tf_keras is None:
+                raise ValueError('You forced tf_keras but do not have tensorflow.keras')
+            keras = tf_keras
+            use_tf_keras = True
+        elif keras_keras is not None:
+            # Use old keras by default, as users may have gone out of their way to install it
+            keras = keras_keras
+            use_tf_keras = False
+            if tf_keras is not None:
+                logging.warning('Model publication will use standalone keras, yet you have tf.keras installed. '
+                                'If you want your model to use tf.keras, use ``force_tf_keras=True``.')
+        elif tf_keras is not None:
+            keras = tf_keras
+            use_tf_keras = True
+        else:
+            raise ValueError('You do not have any version of keras installed.')
 
         # Add model as a file to be sent
         output.add_file(model_path, 'model')
@@ -82,36 +103,33 @@ class KerasModel(BasePythonServableModel):
             model.load_weights(model_path)
 
         # Get the inputs of the model
-        output['servable']['methods']['run']['input'] = output.format_layer_spec(model.input_shape)
-        output['servable']['methods']['run']['output'] = output.format_layer_spec(
-            model.output_shape)
+        output.servable.methods['run'].input = output.format_layer_spec(model.input_shape)
+        output.servable.methods['run'].output = output.format_layer_spec(model.output_shape)
         if output_names is not None:
-            output['servable']['methods']['run']['method_details']['classes'] = output_names
+            output.servable.methods['run'].method_details['classes'] = output_names
 
         # Get a full description of the model. Limit summary to _summary_limit in length
-        output.summary = ""
+        _summary_tmp = []
+        model.summary(print_fn=_summary_tmp.append)
+        summary = "\n".join(_summary_tmp)
+        summary = (summary[:_summary_limit] + '<<TRUNCATED>>') if len(summary) > _summary_limit else summary
 
-        def capture_summary(x):
-            output.summary += x + "\n"
-
-        model.summary(print_fn=capture_summary)
-        output.summary = (output.summary[:_summary_limit] + '<<TRUNCATED>>') \
-            if len(output.summary) > _summary_limit else output.summary
-
-        output['servable']['model_summary'] = output.summary
-        output['servable']['model_type'] = 'Deep NN'
+        output.servable.model_summary = summary
+        output.servable.model_type = 'Deep NN'
 
         # Add keras as a dependency
-        if not keras_version.endswith("-tf"):
+        keras_version = keras.__version__
+        _keras_version_tuple = tuple(int(i) for i in keras_version.rstrip("-tf").split("."))
+        if not (keras_version.endswith("-tf") or use_tf_keras):
             output.add_requirement('keras', keras_version)
         output.add_requirement('h5py', 'detect')
 
         # Detect backend and get its version
-        _detect_backend(output)
+        _detect_backend(keras, output)
 
         return output
 
-    def format_layer_spec(self, layers):
+    def format_layer_spec(self, layers) -> ArgumentTypeMetadata:
         """Make a description of a list of input or output layers
 
         Args:
@@ -120,10 +138,12 @@ class KerasModel(BasePythonServableModel):
             (dict) Description of the inputs / outputs
         """
         if isinstance(layers, tuple):
-            return compose_argument_block("ndarray", "Tensor", shape=list(layers))
+            return ArgumentTypeMetadata.parse_obj(compose_argument_block("ndarray", "Tensor", shape=list(layers)))
         else:
-            return compose_argument_block("tuple", "Tuple of tensors",
-                                          element_types=[self.format_layer_spec(i) for i in layers])
+            return ArgumentTypeMetadata.parse_obj(
+                compose_argument_block("tuple", "Tuple of tensors",
+                                       element_types=[self.format_layer_spec(i) for i in layers])
+            )
 
     def add_custom_object(self, name, custom_object):
         """Add a custom layer to the model specification
@@ -144,11 +164,9 @@ class KerasModel(BasePythonServableModel):
         module = custom_object.__module__
 
         # Add the layer to the model definition
-        if 'options' not in self._output['servable']:
-            self['servable']['options'] = {}
-        if 'custom_objects' not in self['servable']['options']:
-            self['servable']['options']['custom_objects'] = {}
-        self['servable']['options']['custom_objects'][name] = '{}.{}'.format(module, object_name)
+        if 'custom_objects' not in self.servable.options:
+            self.servable.options['custom_objects'] = {}
+        self.servable.options['custom_objects'][name] = '{}.{}'.format(module, object_name)
 
     def _get_handler(self):
         return "keras.KerasServable"
